\version "2.24.1"

spacing = #(define-scheme-function (d) (rational?)
  (if final
    #{ \context { \Score \override SpacingSpanner.base-shortest-duration = #(ly:make-moment d) } #}
  ))

spacer = #(define-music-function (upbeat m) (ly:music? ly:music?)
  (let* ((core  (ly:moment-sub (ly:music-length m) (ly:music-length upbeat)))
         (num   (ly:moment-main-numerator core))
         (denom (ly:moment-main-denominator core)))
    #{ #upbeat \set Timing.measurePosition = #(ly:make-moment 0) \scaleDurations #(cons num denom) s1 #}
  ))

#(define (incrementBarNumber context)
  (let* ((score (ly:context-find context 'Score)))
    (ly:context-set-property! score 'currentBarNumber
      (1+ (ly:context-property score 'currentBarNumber)))))

decrementBarNumber = #(define-scheme-function (count) (integer?)
  (lambda (context)
  (let* ((score (ly:context-find context 'Score)))
    (ly:context-set-property! score 'currentBarNumber
      (- (ly:context-property score 'currentBarNumber) count)))))

KeepEmptyStaves = \with { \override VerticalAxisGroup.remove-empty = ##f }

\layout {
  staffspace = #(ly:output-def-lookup $defaultpaper 'staff-space)
  indent = #(* 24 staffspace)
  \override AccidentalSuggestion.avoid-slur = #'outside
  \override Beam.auto-knee-gap = #+inf.0
  \override DynamicTextSpanner.style = #'none
  \override LaissezVibrerTie.details.note-head-gap = -2
  \override LaissezVibrerTie.extra-offset = #'(2 . 0)
  \override LaissezVibrerTie.X-extent = #'(0 . 0)

  \context {
    \Score
    \consists \Bar_Count_engraver
    barNumberVisibility = #first-bar-number-invisible
    \accepts Structure
  }
  \context { \StaffGroup \override SystemStartBracket.collapse-height = 0 }
  \context { \Staff extraNatural = ##f printKeyCancellation = ##f \consists \SectionRecv }
  \context {
    \Voice
    \override Script.script-priority = 190 % move trills inside partcombine texts
    \override DynamicText.extra-spacing-width = #'(+inf.0 . -inf.0)
    \hide TupletBracket
  }
  \context {
    \Devnull \name "Structure"
    \consists "Bar_engraver"
    \consists \SectionBcast
  }

  % Groups
  % Create StaffGroup contexts
  $@(map groupcxt groups)

  % Add details
  \context {
    \Violins
    instrumentName = "Violins"
    shortInstrumentName = "Vln"
  }
  \context {
    \Violas
    instrumentName = "Violas"
    shortInstrumentName = "Vla"
  }
  \context {
    \Flutes
    instrumentName = "Flutes"
    shortInstrumentName = "Fl"
  }
  \context {
    \Oboes
    instrumentName = "Oboes"
    shortInstrumentName = "Ob"
  }
  \context {
    \Bassoons
    instrumentName = "Bassoons"
    shortInstrumentName = "Bn"
  }
  \context {
    \Horns
    shortInstrumentName = "Hn"
  }

  % Instruments pairs
  % Create Staff contexts
  $@(map paircxt pairs)

  % Add details
  \context {
    \ViolaIandII
    instrumentName = \markup \pairinsts "I" "II" "Violas"
    shortInstrumentName = "Vla"
    aDueText = \markup \whiteout "unis"
    \clef "alto"
  }
  \context {
    \FluteIandII
    instrumentName = \markup \pairinsts "I" "II" "Flutes"
    shortInstrumentName = "Fl"
    \clef "treble"
    \consists \Print_directions
  }
  \context {
    \OboeIandII
    instrumentName = \markup \pairinsts "I" "II" "Oboes"
    shortInstrumentName = "Ob"
    \clef "treble"
    \consists \Print_directions
  }
  \context {
    \CorAnglaisIandII
    instrumentName = \markup \pairinsts "I" "II" "Cors Anglais"
    shortInstrumentName = "CorA"
    instrumentTransposition = f
    \clef "treble"
    \autoTranspose
    \consists \Print_directions
  }
  \context {
    \ClarinetIandII
    instrumentName = "Clarinets"
    shortInstrumentName = "Cl"
    \clef "treble"
    \autoTranspose
    \consists \Transposing_instrument_name
    \consists \Print_directions
  }
  \context {
    \BassoonIandII
    instrumentName = \markup \pairinsts "I" "II" "Bassoons"
    shortInstrumentName = "Bn"
    \clef "bass"
    \consists \Print_directions
  }
  \context {
    \TrumpetIandII
    instrumentName = "Trumpets"
    shortInstrumentName = "Tpt"
    \clef "treble"
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
    \consists \Print_directions
  }
  \context {
    \HornIandII
    instrumentName = "Horns"
    shortInstrumentName = "Hn"
    \clef "treble"
    \hide ClefModifier
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
    \consists \Print_directions
  }
  \context {
    \HornIIIandIV
    instrumentName = "Horns"
    shortInstrumentName = "Hn"
    \clef "treble"
    \hide ClefModifier
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
    \consists \Print_directions
  }

  % Single instruments
  % Create Staff contexts
  $@(map staffcxt instruments)

  % Add details
  \context {
    \Flute
    instrumentName = "Flute"
    shortInstrumentName = "Fl"
    \clef "treble"
  }
  \context {
    \Oboe
    instrumentName = "Oboe"
    shortInstrumentName = "Ob"
    \clef "treble"
  }
  \context {
    \Bassoon
    instrumentName = "Bassoon"
    shortInstrumentName = "Bn"
    \clef "bass"
  }
  \context {
    \Violin
    \name "ViolinSolo"
    instrumentName = "Violin Solo"
    shortInstrumentName = "VlnS"
    \clef "treble"
  }
  \context {
    \Violin
    \name "ViolinISolo"
    instrumentName = \markup \center-column { "Violin I" "Solo" }
    shortInstrumentName = \markup \hcenter-in #2 "S"
    \clef "treble"
  }
  \context {
    \Violin
    \name "ViolinIISolo"
    instrumentName = \markup \center-column { "Violin II" "Solo" }
    shortInstrumentName = \markup \hcenter-in #2 "S"
    \clef "treble"
  }
  \context {
    \Viola
    instrumentName = "Viola"
    shortInstrumentName = "Vla"
    \clef "alto"
  }
  \context {
    \Viola
    \name "ViolaSolo"
    instrumentName = "Viola Solo"
    shortInstrumentName = "VlaS"
    \clef "alto"
  }
  \context {
    \Cello
    instrumentName = "Cello"
    shortInstrumentName = "Vlc"
    \clef "bass"
  }
  \context {
    \Cello
    \name "CelloSolo"
    instrumentName = "Cello Solo"
    shortInstrumentName = "VlcS"
    \clef "bass"
  }
  \context {
    \Bass
    instrumentName = "Bass"
    shortInstrumentName = "Db"
    \clef "bass"
    instrumentTransposition = c
    \autoTranspose
  }
  \context {
    \Bass
    \name "BassSolo"
    instrumentName = "Violone Solo"
    shortInstrumentName = "Vne"
    \clef "bass"
  }
  \context {
    \CB
    instrumentName = "Cello, Bass"
    shortInstrumentName = \markup \center-column { "Vlc" "Db" }
    \clef "bass"
    aDueText   = \markup \whiteout "Tutti"
    soloText   = \markup \whiteout "Vlc"
    soloIIText = \markup \whiteout "Db"
    \override VerticalAxisGroup.remove-layer = 2
  }
  \context {
    \CBB
    instrumentName = \markup \center-column { "Cello, Bass" "Bassoon" }
    shortInstrumentName = \markup \center-column { "Vlc" "Db Bn" }
    \clef "bass"
    aDueText   = \markup \whiteout "Tutti"
    soloText   = \markup \whiteout "Vlc"
    soloIIText = \markup \whiteout "Db Bn"
    \override VerticalAxisGroup.remove-layer = 2
  }
  \context {
    \Timpani
    instrumentName = "Timpani"
    shortInstrumentName = "Timp"
    \clef "bass"
    \remove "Key_engraver"
    \consists \instrumentTuning
  }
  \context {
    \Trumpet
    instrumentName = "Trumpet"
    shortInstrumentName = "Tpt"
    \clef "treble"
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
  }
  \context {
    \Horn
    instrumentName = "Horn"
    shortInstrumentName = "Hn"
    \clef "treble"
    \hide ClefModifier
    \autoTranspose
    \remove "Key_engraver"
    \consists \Transposing_instrument_name
  }

  % First, second etc.
  \context { \FluteI       instrumentName=\markup \oneinst "I"   \clef "treble" }
  \context { \FluteII      instrumentName=\markup \oneinst "II"  \clef "treble" }
  \context { \OboeI        instrumentName=\markup \oneinst "I"   \clef "treble" }
  \context { \OboeII       instrumentName=\markup \oneinst "II"  \clef "treble" }
  \context { \BassoonI     instrumentName=\markup \oneinst "I"   \clef "bass" }
  \context { \BassoonII    instrumentName=\markup \oneinst "II"  \clef "bass" }
  \context { \HornI        instrumentName=\markup \oneinst "I"   \clef "treble"
                           \hide ClefModifier \autoTranspose  \remove "Key_engraver"  }
  \context { \HornII       instrumentName=\markup \oneinst "II"  \clef "treble"
                           \hide ClefModifier \autoTranspose  \remove "Key_engraver"  }
  \context { \HornIII      instrumentName=\markup \oneinst "III" \clef "treble"
                           \hide ClefModifier \autoTranspose  \remove "Key_engraver"  }
  \context { \HornIV       instrumentName=\markup \oneinst "IV"  \clef "treble"
                           \hide ClefModifier \autoTranspose  \remove "Key_engraver"  }
  \context { \ViolinI      instrumentName=\markup \oneinst "I"   \clef "treble"
                           shortInstrumentName=\markup \hcenter-in #2 "I"  }
  \context { \ViolinII     instrumentName=\markup \oneinst "II"  \clef "treble"
                           shortInstrumentName=\markup \hcenter-in #2 "II" }
  \context { \ViolinIII    instrumentName=\markup \oneinst "III"  \clef "treble"
                           shortInstrumentName=\markup \hcenter-in #2 "III" }
  \context { \ViolinIV     instrumentName=\markup \oneinst "IV"  \clef "treble"
                           shortInstrumentName=\markup \hcenter-in #2 "IV" }
  \context { \ViolaI       instrumentName=\markup \oneinst "I"   \clef "alto"
                           shortInstrumentName=\markup \hcenter-in #2 "I"  }
  \context { \ViolaII      instrumentName=\markup \oneinst "II"  \clef "alto"
                           shortInstrumentName=\markup \hcenter-in #2 "II" }
}

breaklayout = #(define-scheme-function (weight mvt) ((number? 0.25) symbol?)
  (if final
    (let* ((movement (ly:parser-lookup mvt))
           (music    (assoc-ref movement 'structure))
           (global   (ly:run-translator (ly:score-music (scorify-music music)) barlistlayout))
           (barlist  (ly:context-property global 'barlist)))
      (if debug (format #t "\n~a" barlist))
      #{ \context { \Score \consists #(Break_engraver barlist weight) \consists "Bar_engraver" } #}
  )))

barlistlayout = \layout {
  fonts        = #(ly:output-def-lookup $defaultpaper 'fonts)
  output-scale = #(ly:output-def-lookup $defaultpaper 'output-scale)
  \context { \Score \consists #(Bar_List_engraver) \consists "Bar_engraver" }
}

#(hierarchy $defaultlayout)
